/*
* Exercise on C18 compiler version 2.40; IDE is MPLAB 7.41
* Print real-time Temperature temperature [C] and humidity [%RH] to CM50010ACT6 with a funny icon
* Hardware :	PIC18-LF4550-STK1, PIC18LF4550 at Vdd=3.3V
* File:			..\sht10_dhc1600\SHTxx_main.c
* Functions: 	Example software for SHT10 digital temperature & humidity sensor
* Version:		1.0 (first version) on 21 Oct 2006
* Programmer: 	John Leung at www.TechToys.com.hk		
* Downloaded from www.microchipc.com - see site for more sample PIC C code
*/

/*
********************************************************************************************************************
* 31 July 2007
* Program modified for HDC1600 65k color LCD controller
********************************************************************************************************************
*/

#include <p18cxxx.h>
#include "delay.h"
#include "SHT1x.h"
#include "hdc1600.h"
#include "icon.h"

/*
********************************************************************************************************************
*												Pragma Section
*
* Configuration bits under MPLAB v7.41 subject to changes on every little change on project settings.
* So #pragma config statments used for configuration bits instead of using Configuration bits tab in MPLAB
* Note: Refer to pdf document "mcc18\doc\MPLAB-C18-Addendum.pdf" under mcc18 installation directory for config
* 		variables available
********************************************************************************************************************
*/
#pragma config PLLDIV = 5			// This value has to be 5 for 20Mhz xtal as PLL accepts fixed input of 4MHz only
#pragma config USBDIV = 2			// Clock source from 96MHz PLL/2, therefore USB clock is 48MHz in full speed
#pragma config FOSC   = HSPLL_HS	// HS oscillator, PLL enabled
#pragma config CPUDIV = OSC4_PLL6	// 96MHz PLL Src/6 => a CPU clock of 16MHz achieved for Vdd=3.3V
#pragma config PWRT   = ON			// PowerUp timer ON
#pragma config WDT    = OFF			// watchdog timer off
#pragma config LVP    = OFF			// low level voltage progamming OFF
#pragma config BOR    = OFF			// brown out detect off
#pragma config DEBUG  = ON			// Debug ON
#pragma config PBADEN = OFF 		// PORTB<4:0> as digital IO on reset
#pragma config STVREN = ON  		// Stack overflow reset ON
#pragma config VREGEN = OFF			// Disable internal 3.3V regulator!
#pragma config MCLRE  = ON			// Enable MCLRE for reset function


typedef union
{
	unsigned int i;
	float f;
} value;

/*
*********************************************************************************************************
*                         				CALCULATE RH AND TEMPERATURE
*
* Description :	This function returns temperature and relative humidity in floating point values
* Arguments   : float '*p_humidity'		pointer to humidity
*				float '* p_temperature' pointer to temperature
*				
* Returns     : none
* Notes		  : humidity in 12 bit raw data and temperature in 14 bit raw data passed to this
*				function, where at the same ram location, floating point values of temperature and
*				relative humidity value will be returned. 
*				Reference: an Keil C example for 8051 from Sensirion at www.sensirion.com
*********************************************************************************************************
*/	

void calc_sht1x(float *p_humidity ,float *p_temperature)
// calculates temperature [C] and humidity [%RH] 
// input :  humi [Ticks] (12 bit) 
//          temp [Ticks] (14 bit)
// output:  humi [%RH]
//          temp [deg Cel]
{ float rh=*p_humidity;             				// rh:      Humidity [Ticks] 12 Bit 
  float t=*p_temperature;           				// t:       Temperature [Ticks] 14 Bit
  float rh_lin;                     				// rh_lin:  Humidity linear
  float rh_true;                    				// rh_true: Temperature compensated humidity
  float t_C;                        				// t_C   :  Temperature [C]

  t_C=t*0.01 - 40;                  				//calc. temperature from ticks to [deg Cel]	
  rh_lin=0.0405*rh - 0.0000028*rh*rh - 4.0;     	//calc. humidity from ticks to [%RH]
  rh_true=(t_C-25)*(0.01+0.00008*rh)+rh_lin;   		//calc. temperature compensated humidity [%RH]
  if(rh_true>100)rh_true=100;       				//cut if the value is outside of
  if(rh_true<0.1)rh_true=0.1;       				//the physical possible range

  *p_temperature=t_C;               				//return temperature [deg Cel]
  *p_humidity=rh_true;              				//return humidity[%RH]
}


/*
*********************************************************************************************************
*                        CONVERT A FLOATING POINT NUMBER TO STRING WITH 1 DECIMAL PLACE
*
* Description :	This function converts a floating point to a null terminated string 
*				with 1 decimal place.
*			
*				Examples:
*					
*				float f = 9.567;
*				ftoa(&s[0], f);		//s[]={'9','.','5', 0}
*				float f = -0.189;
*				ftoa(&s[0], f);  	//s[]={'-', '0', '.', '1', 0}
* Arguments   : 'unsigned char* buf' 	is the pointer to the string holding the conversion result
*				'float f'				is the input floating point
* Returns     : Returns the string with unsigned char* buf pointing to.
* Notes		  : This routine modified from itoa10() in ..\sample\misc folder of ht-picc
*				If more decimal places required, modify the last section of this code
*				Range of f in (-3,276.7, 3,276.7)
*				This function does print result like 0.0.
*********************************************************************************************************
*/
void ftoa(unsigned char *buf, float f) {
	unsigned int rem;
	unsigned char *s,length=0;
	int i;

	i = (int)((float)f*10);

	s = buf;
	if (i == 0){ 		//print 0.0 with null termination here
		*s++ = '0';
		*s++ = '.';
		*s++ = '0'; 
		*s=0; 			//null terminate the string
	} else {	
		if (i < 0) {
			*buf++ = '-';
			s = buf;
			i = -i;
		}
		//while-loop to "decode" the long integer to ASCII by append '0', string in reverse manner
		//If it is an integer of 124 -> string = {'4', '2', '1'}
		while (i) {
			++length;
			rem = i % 10;
			*s++ = rem + '0';
			i /= 10;
		}
		//reverse the string in this for-loop, string became {'1', '2', '4'} after this for-loop
		for(rem=0; ((unsigned char)rem)<length/2; rem++) {
			*(buf+length) = *(buf+((unsigned char)rem));
			*(buf+((unsigned char)rem)) = *(buf+(length-((unsigned char)rem)-1));
			*(buf+(length-((unsigned char)rem)-1)) = *(buf+length);
		}

		/* Take care of the special case of 0.x if length ==1*/	
		if(length==1) {
			*(buf+2) = *buf;
			*buf = '0';
			*(buf+1) = '.';
			*(s+2)=0; 			//null terminate
		} else {
			*(buf+length) = *(buf+length-1);
			*(buf+length-1)='.';
			*(s+1)=0; 			//null terminate
		}
	}
}


/*
********************************************************************************************************************
*													MAIN
********************************************************************************************************************
*/
void main(void)
{
// Sample program that shows how to use SHT10 functions and got the data displayed on CM60010ACT6 color LCD
// 1. Initialize the LCD and Display the background picture and text
// 2. Enable backlight
// 3. Connection reset (sensor init)
// 4. Measure humidity [ticks](12 bit) and temperature [ticks](14 bit)
// 5. Calculate humidity [%RH] and temperature [C]
// 6. Convert temperature in floating point to null terminated string for printing
// 7. Clear the background for digits
// 8. Print the digits of temperature
// 9. Repeat for RH value
// 10. Delay for 1s and loop all over

	value humi_val, temp_val;
	unsigned char error, checksum;
	unsigned char x,y;
	unsigned char s[6];
	
	ADCON1 = 0x0F;					//All digital operation	for RA2 & RA0 pins being digital for SHT10 sensor

	cDispInit();													//initialize the CM50010ACT6 module
	cDispClr(RGB(31,63,31));											//clear screen in white
	cDisprPatAt(19,10, &icon[0]);										//print the icon
	cDisprStrAt(4, 110, "Temperature:     C", 1, RGB(0,0,0));			//print the fixed text
	cDisprStrAt(4, 130, "Relative Humidity:     %", 1, RGB(0,0,0));
	cDispBackLite(ENABLE);												//enable backlight

	s_softreset();														//reset the SHT10 sensor
	DelayMs(20);

	for(;;){
	error=0;
    error+=s_measure((unsigned char*)&humi_val.i,&checksum,HUMI);  	//measure humidity
    error+=s_measure((unsigned char*)&temp_val.i,&checksum,TEMP);  	//measure temperature
    if(error!=0) s_connectionreset();								//in case of an error: connection reset               
    else
	{
      	humi_val.f=(float)humi_val.i;								//converts integer to float
      	temp_val.f=(float)temp_val.i;								//converts integer to float
      	calc_sht1x(&humi_val.f,&temp_val.f);						//calculate humidity, temperature
		ftoa(s, temp_val.f);										//convert floating point to string in 1 dec place
		for(y=0;y<9;y++)											//clear and paint the background for temperature
		{
			for (x=0;x<22;x++)cDispSetPixel(76+x, 108+y, RGB(31,63,31));
		}
		cDispStrAt(76,110,s,1,RGB(0,0,0));						//print temperature string
		ftoa(s, humi_val.f);									//convert floating point to string in 1 dec place
		for(y=0;y<9;y++)										//clear and paint the background for RH
		{
			for (x=0;x<22;x++)cDispSetPixel(98+x, 128+y, RGB(31,63,31));
		}
		cDispStrAt(98,130,s,1,RGB(0,0,0));						//print RH string
    }
      //----------wait approx. 1s to avoid heating up SHTxx--------------    
	  for(x=0;x<5;x++) DelayMs(200);       
	}
}
